# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import positions.fields


def add_test_data(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    MigrationTest = apps.get_model("migration", "MigrationTest")
    test_record = MigrationTest.objects.create(name='Test Name', age=99, favorite_color='Red')

class Migration(migrations.Migration):

    dependencies = [
        ('migration', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(add_test_data),
    ]





from django.db import models

from positions import PositionField


class SubUnit(models.Model):
    name = models.CharField(max_length=100)


class Task(models.Model):
    """
    Base class for lessons/exercises - ordered items within a sub-unit
    """
    sub_unit = models.ForeignKey(SubUnit)
    title = models.CharField(max_length=100)
    position = PositionField(collection='sub_unit', parent_link='task_ptr')


class Lesson(Task):
    text = models.CharField(max_length=100)


class Exercise(Task):
    description = models.CharField(max_length=100)



import time
import doctest
import unittest

from positions.examples.lists.models import List, Item

from django.test import TestCase

class GenericTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # @unittest.skip("Some reason. If you are reading this in a test run someone did not fill this in.")
    def test_doctests_standin(self):
        # This code just contains the old doctests for this module. They should be most likely split out into their own
        # tests at some point.
        self.l = List.objects.create(name='To Do')

        # create a couple items using the default position
        result = self.l.items.create(name='Write Tests').name
        expected_result = 'Write Tests'
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position'))
        expected_result = [(u'Write Tests', 0)]
        self.assertEqual(result, expected_result)

        result = self.l.items.create(name='Exercise').name
        expected_result = 'Exercise'
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Write Tests', 0), (u'Exercise', 1)]
        self.assertEqual(result, expected_result)

        # create an item with an explicit position
        result = self.l.items.create(name='Learn to spell Exercise', position=0).name
        expected_result = 'Learn to spell Exercise'
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Learn to spell Exercise', 0), (u'Write Tests', 1), (u'Exercise', 2)]
        self.assertEqual(result, expected_result)

        # save an item without changing it's position
        self.exercise = self.l.items.order_by('-position')[0]
        self.exercise.name = 'Exercise'
        self.exercise.save()

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Learn to spell Exercise', 0), (u'Write Tests', 1), (u'Exercise', 2)]
        self.assertEqual(result, expected_result)

        # delete an item
        self.learn_to_spell = self.l.items.order_by('position')[0]
        self.learn_to_spell.delete()

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Write Tests', 0), (u'Exercise', 1)]
        self.assertEqual(result, expected_result)

        # create a couple more items
        result = self.l.items.create(name='Drink less Coke').name
        expected_result = 'Drink less Coke'
        self.assertEqual(result, expected_result)

        result = self.l.items.create(name='Go to Bed').name
        expected_result = 'Go to Bed'
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Write Tests', 0), (u'Exercise', 1), (u'Drink less Coke', 2), (u'Go to Bed', 3)]
        self.assertEqual(result, expected_result)

        # move item to end using None
        self.write_tests = self.l.items.order_by('position')[0]
        self.write_tests.position = None
        self.write_tests.save()
        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Exercise', 0), (u'Drink less Coke', 1), (u'Go to Bed', 2), (u'Write Tests', 3)]
        self.assertEqual(result, expected_result)

        # move item using negative index
        self.write_tests.position = -3
        self.write_tests.save()
        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Exercise', 0), (u'Write Tests', 1), (u'Drink less Coke', 2), (u'Go to Bed', 3)]
        self.assertEqual(result, expected_result)

        # move item to position
        self.write_tests.position = 2
        self.write_tests.save()

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Exercise', 0), (u'Drink less Coke', 1), (u'Write Tests', 2), (u'Go to Bed', 3)]
        self.assertEqual(result, expected_result)

        # move item to beginning
        self.sleep = self.l.items.order_by('-position')[0]
        self.sleep.position = 0
        self.sleep.save()

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Go to Bed', 0), (u'Exercise', 1), (u'Drink less Coke', 2), (u'Write Tests', 3)]
        self.assertEqual(result, expected_result)

        # check auto_now updates
        time.sleep(1)  # sleep to guarantee updated time increases
        sleep_updated, exercise_updated, eat_better_updated, write_tests_updated = [i.updated for i in self.l.items.order_by('position')]
        self.eat_better = self.l.items.order_by('-position')[1]
        self.eat_better.position = 1
        self.eat_better.save()
        self.todo_list = list(self.l.items.order_by('position'))

        result = sleep_updated == self.todo_list[0].updated
        expected_result = True
        self.assertEqual(result, expected_result)

        result = eat_better_updated < self.todo_list[1].updated
        expected_result = True
        self.assertEqual(result, expected_result)

        result = exercise_updated < self.todo_list[2].updated
        expected_result = True
        self.assertEqual(result, expected_result)

        result = write_tests_updated == exercise_updated
        expected_result = True
        self.assertEqual(result, expected_result)

        # create an item using negative index
        # http://github.com/jpwatts/django-positions/issues/#issue/5
        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Go to Bed', 0), (u'Drink less Coke', 1), (u'Exercise', 2), (u'Write Tests', 3)]
        self.assertEqual(result, expected_result)

        self.fix_issue_5 = Item(list=self.l, name="Fix Issue #5")
        result = self.fix_issue_5.position
        expected_result = -1
        self.assertEqual(result, expected_result)

        self.fix_issue_5.position = -2
        result = self.fix_issue_5.position
        expected_result = -2
        self.assertEqual(result, expected_result)

        self.fix_issue_5.save()
        result = self.fix_issue_5.position
        expected_result = 3
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Go to Bed', 0), (u'Drink less Coke', 1), (u'Exercise', 2), (u'Fix Issue #5', 3), (u'Write Tests', 4)]
        self.assertEqual(result, expected_result)

        # Try again, now that the model has been saved.
        self.fix_issue_5.position = -2
        self.fix_issue_5.save()
        result = self.fix_issue_5.position
        expected_result = 3
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Go to Bed', 0), (u'Drink less Coke', 1), (u'Exercise', 2), (u'Fix Issue #5', 3), (u'Write Tests', 4)]
        self.assertEqual(result, expected_result)

        # create an item using with a position of zero
        # http://github.com/jpwatts/django-positions/issues#issue/7
        self.item0 = self.l.items.create(name="Fix Issue #7", position=0)
        result =  self.item0.position
        expected_result = 0
        self.assertEqual(result, expected_result)

        result = list(self.l.items.values_list('name', 'position').order_by('position'))
        expected_result = [(u'Fix Issue #7', 0), (u'Go to Bed', 1), (u'Drink less Coke', 2), (u'Exercise', 3), (u'Fix Issue #5', 4), (u'Write Tests', 5)]
        self.assertEqual(result, expected_result)



from django.test import TestCase
import doctest
import unittest
import pprint
from positions.examples.photos.forms import PhotoForm
from positions.examples.photos.models import Album, Photo

class PhotosTestCase(TestCase):
    def setUp(self):
        self.album = Album.objects.create(name="Vacation")
        self.bahamas = self.album.photos.create(name="Bahamas")
        self.bahamas_id = self.bahamas.id
        self.assertEqual(self.bahamas.position, 0)

        self.jamaica = self.album.photos.create(name="Jamaica")
        self.jamaica_id = self.jamaica.id
        self.assertEqual(self.jamaica.position, 0)

        self.grand_cayman = self.album.photos.create(name="Grand Cayman")
        self.grand_cayman_id = self.grand_cayman.id
        self.assertEqual(self.grand_cayman.position, 0)

        self.cozumel = self.album.photos.create(name="Cozumel")
        self.cozumel_id = self.cozumel.id
        self.assertEqual(self.cozumel.position, 0)

    def refresh(self):
        self.bahamas = Photo.objects.get(id=self.bahamas_id)
        self.jamaica = Photo.objects.get(id=self.jamaica_id)
        self.grand_cayman = Photo.objects.get(id=self.grand_cayman_id)
        self.cozumel = Photo.objects.get(id=self.cozumel_id)

    def tearDown(self):
        Album.objects.all().delete()

    def test_reordered_positions(self):
        ordered_by_position = list(self.album.photos.order_by('position').values_list('name', 'position'))
        expected_order = [(u'Cozumel', 0), (u'Grand Cayman', 1), (u'Jamaica', 2), (u'Bahamas', 3)]
        self.assertEqual(
            ordered_by_position,
            expected_order
        )

    def test_renamed_positions(self):
        self.refresh()
        new_name = 'Cozumel, Mexico'
        self.cozumel.name = new_name
        self.cozumel.save(update_fields=['name'])
        self.refresh()
        self.assertEqual(self.cozumel.name, new_name)
        self.assertEqual(self.cozumel.position, 0)

        self.jamaica.name = "Ocho Rios, Jamaica"
        self.jamaica.save(update_fields=['name', 'position'])
        self.refresh()
        self.assertEqual(self.jamaica.position, 2)

        self.jamaica.position = -1
        self.jamaica.save(update_fields=['name', 'position'])
        self.refresh()
        self.assertEqual(self.jamaica.position, 3)

    def test_form_renamed_position(self):
        self.refresh()
        grand_cayman_form = PhotoForm(dict(name="Georgetown, Grand Cayman"), instance=self.grand_cayman)
        grand_cayman_form.save()
        self.refresh()
        self.assertEqual(self.grand_cayman.position, 1)



import doctest
import unittest

from positions.examples.todo.models import Item

from django.test import TestCase

class GenericTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # @unittest.skip("Some reason. If you are reading this in a test run someone did not fill this in.")
    def test_doctests_standin(self):
        # This code just contains the old doctests for this module. They should be most likely split out into their own
        # tests at some point.
        result = Item.objects.position_field_name
        expected_result = 'index'
        self.assertEqual(result, expected_result)

        result = Item.objects.all().position_field_name
        expected_result = 'index'
        self.assertEqual(result, expected_result)

        result = Item.objects.create(description="Add a `reposition` method").description
        expected_result = 'Add a `reposition` method'
        self.assertEqual(result, expected_result)

        result = Item.objects.create(description="Write some tests").description
        expected_result = 'Write some tests'
        self.assertEqual(result, expected_result)

        result = Item.objects.create(description="Push to GitHub").description
        expected_result =  'Push to GitHub'
        self.assertEqual(result, expected_result)

        result =  list(Item.objects.order_by('index').values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Write some tests',), (u'Push to GitHub',)]
        self.assertEqual(result, expected_result)

        self.alphabetized = Item.objects.order_by('description')
        result = list(self.alphabetized.values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Push to GitHub',), (u'Write some tests',)]
        self.assertEqual(result, expected_result)

        result = self.alphabetized.position_field_name
        expected_result = 'index'
        self.assertEqual(result, expected_result)

        self.repositioned = self.alphabetized.reposition(save=False)
        result = list(self.repositioned.values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Push to GitHub',), (u'Write some tests',)]
        self.assertEqual(result, expected_result)

        # Make sure the position wasn't saved
        result = list(Item.objects.order_by('index').values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Write some tests',), (u'Push to GitHub',)]
        self.assertEqual(result, expected_result)

        self.repositioned = self.alphabetized.reposition()
        result = list(self.repositioned.values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Push to GitHub',), (u'Write some tests',)]
        self.assertEqual(result, expected_result)

        result = list(Item.objects.order_by('index').values_list('description'))
        expected_result = [(u'Add a `reposition` method',), (u'Push to GitHub',), (u'Write some tests',)]
        self.assertEqual(result, expected_result)

        self.item = Item.objects.order_by('index')[0]
        result = self.item.description
        expected_result = 'Add a `reposition` method'
        self.assertEqual(result, expected_result)

        result = self.item.index
        expected_result = 0

        self.item.index = -1
        self.item.save()

        # Make sure the signals are still connected
        result = list(Item.objects.order_by('index').values_list('description'))
        expected_result = [(u'Push to GitHub',), (u'Write some tests',), (u'Add a `reposition` method',)]
        self.assertEqual(result, expected_result)

        result = [i.index for i in Item.objects.order_by('index')]
        expected_result = [0, 1, 2]
        self.assertEqual(result, expected_result)


        # Add an item at position zero
        # http://github.com/jpwatts/django-positions/issues/#issue/7

        self.item0 = Item(description="Fix Issue #7")
        self.item0.index = 0
        self.item0.save()

        result = list(Item.objects.values_list('description', 'index').order_by('index'))
        expected_result = [(u'Fix Issue #7', 0), (u'Push to GitHub', 1), (u'Write some tests', 2), (u'Add a `reposition` method', 3)]
        self.assertEqual(result, expected_result)



import doctest
import unittest

from django.db import models

from positions.examples.restaurants.models import Menu, Food, Drink

from django.test import TestCase

class GenericTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # @unittest.skip("Some reason. If you are reading this in a test run someone did not fill this in.")
    def test_doctests_standin(self):
        # This code just contains the old doctests for this module. They should be most likely split out into their own
        # tests at some point.
        self.romanos = Menu.objects.create(name="Romano's Pizza")

        self.pizza = Food.objects.create(menu=self.romanos, name="Pepperoni")
        result = self.pizza.position
        expected_result = 0
        self.assertEqual(result, expected_result)

        self.wine = Drink.objects.create(menu=self.romanos, name="Merlot")
        result = self.wine.position
        expected_result = 0
        self.assertEqual(result, expected_result)

        self.spaghetti = Food.objects.create(menu=self.romanos, name="Spaghetti & Meatballs")
        result = self.spaghetti.position
        expected_result = 1
        self.assertEqual(result, expected_result)

        self.soda = Drink.objects.create(menu=self.romanos, name="Coca-Cola")
        result = self.soda.position
        expected_result = 1
        self.assertEqual(result, expected_result)



from django.db import models

# Create your models here.
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models

from positions.fields import PositionField


class MigrationTest(models.Model):
    name = models.CharField(max_length=80)
    age = models.IntegerField(null=True, blank=True)
    favorite_color = models.CharField(max_length=255, null=True, blank=True)
    position = PositionField(collection=('name', 'age'))

    def __unicode__(self):
        return self.name


from django.db import models

import positions


class Item(models.Model):
    description = models.CharField(max_length=50)

    # I'm calling the PositionField "index" to make sure any internal code that
    # relies on a PositionField being called "position" will break.
    # https://github.com/jpwatts/django-positions/pull/12
    index = positions.PositionField()

    objects = positions.PositionManager('index')

    def __unicode__(self):
        return self.description



import doctest
import unittest

from django.db import models

from positions import PositionField
from positions.examples.store.models import Product, Category, ProductCategory

from django.test import TestCase

class StoreTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        Category.objects.all().delete()
        Product.objects.all().delete()
        ProductCategory.objects.all().delete()

    # @unittest.skip("Some reason. If you are reading this in a test run someone did not fill this in.")
    def test_doctests_standin(self):
        # This code just contains the old doctests for this module. They should be most likely split out into their own
        # tests at some point.
        self.clothes = Category.objects.create(name="Clothes")
        self.sporting_goods = Category.objects.create(name="Sporting Goods")

        self.bat = Product.objects.create(name="Bat")
        self.bat_in_sporting_goods = ProductCategory.objects.create(product=self.bat, category=self.sporting_goods)

        self.cap = Product.objects.create(name="Cap")
        self.cap_in_sporting_goods = ProductCategory.objects.create(product=self.cap, category=self.sporting_goods)
        self.cap_in_clothes = ProductCategory.objects.create(product=self.cap, category=self.clothes)

        self.glove = Product.objects.create(name="Glove")
        self.glove_in_sporting_goods = ProductCategory.objects.create(product=self.glove, category=self.sporting_goods)

        self.tshirt = Product.objects.create(name="T-shirt")
        self.tshirt_in_clothes = ProductCategory.objects.create(product=self.tshirt, category=self.clothes)

        self.jeans = Product.objects.create(name="Jeans")
        self.jeans_in_clothes = ProductCategory.objects.create(product=self.jeans, category=self.clothes)

        self.jersey = Product.objects.create(name="Jersey")
        self.jersey_in_sporting_goods = ProductCategory.objects.create(product=self.jersey, category=self.sporting_goods)
        self.jersey_in_clothes = ProductCategory.objects.create(product=self.jersey, category=self.clothes)

        self.ball = Product.objects.create(name="Ball")
        self.ball_in_sporting_goods = ProductCategory.objects.create(product=self.ball, category=self.sporting_goods)

        actual_order = list(ProductCategory.objects.filter(category=self.clothes).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'Cap', 0), (u'T-shirt', 1), (u'Jeans', 2), (u'Jersey', 3)]
        self.assertEqual(actual_order, expected_order)

        actual_order = list(ProductCategory.objects.filter(category=self.sporting_goods).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'Bat', 0), (u'Cap', 1), (u'Glove', 2), (u'Jersey', 3), (u'Ball', 4)]
        self.assertEqual(actual_order, expected_order)

        # Moving cap in sporting goods shouldn't effect its position in clothes.

        self.cap_in_sporting_goods.position = -1
        self.cap_in_sporting_goods.save()

        actual_order = list(ProductCategory.objects.filter(category=self.clothes).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'Cap', 0), (u'T-shirt', 1), (u'Jeans', 2), (u'Jersey', 3)]
        self.assertEqual(actual_order, expected_order)

        actual_order = list(ProductCategory.objects.filter(category=self.sporting_goods).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'Bat', 0), (u'Glove', 1), (u'Jersey', 2), (u'Ball', 3), (u'Cap', 4)]
        self.assertEqual(actual_order, expected_order)

        # Deleting an object should reorder both collections.
        self.cap.delete()

        actual_order = list(ProductCategory.objects.filter(category=self.clothes).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'T-shirt', 0), (u'Jeans', 1), (u'Jersey', 2)]
        self.assertEqual(actual_order, expected_order)

        actual_order = list(ProductCategory.objects.filter(category=self.sporting_goods).values_list('product__name', 'position').order_by('position'))
        expected_order = [(u'Bat', 0), (u'Glove', 1), (u'Jersey', 2), (u'Ball', 3)]
        self.assertEqual(actual_order, expected_order)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import positions.fields


class Migration(migrations.Migration):

    dependencies = [
        ('migration', '0002_insert_test_data'),
    ]

    operations = [
        migrations.AddField(
            model_name='migrationtest',
            name='position',
            field=positions.fields.PositionField(default=-1),
            preserve_default=True,
        ),
    ]



from django.test import TestCase
from positions.examples.nodes.models import Node
import doctest
import os
import unittest


class NodesTestCase(TestCase):
    def setUp(self):
        """
        Creates a simple tree::

            parent1
                child2
                child1
                child3
            parent2
                child4
                child5
                child6
        """
        self.parent1 = Node.objects.create(name='Parent 1')
        self.parent2 = Node.objects.create(name='Parent 2')
        self.child1 = self.parent1.children.create(name='Child 1')
        self.child2 = self.parent1.children.create(name='Child 2')
        self.child3 = self.parent1.children.create(name='Child 3')
        self.child2.position = 0
        self.child2.save()
        self.child1 = Node.objects.get(pk=self.child1.pk)
        self.child2 = Node.objects.get(pk=self.child2.pk)
        self.child3 = Node.objects.get(pk=self.child3.pk)

        self.child4 = self.parent2.children.create(name='Child 4')
        self.child5 = self.parent2.children.create(name='Child 5')
        self.child6 = self.parent2.children.create(name='Child 6')

    def tearDown(self):
        Node.objects.all().delete()

    def test_structure(self):
        """
        Tests the tree structure
        """
        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 2', 0), (u'Child 1', 1), (u'Child 3', 2), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_sibling_position(self):
        """
        Set child6 as the first sibling in its branch.
        """
        self.child6.position = 0
        self.child6.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 2', 0), (u'Child 1', 1), (u'Child 3', 2), (u'Child 6', 0), (u'Child 4', 1), (u'Child 5', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_first_child(self):
        """
        Move child2 to make it the first child of parent2
        """
        self.child2.position = 0
        self.child2.parent = Node.objects.get(pk=self.parent2.pk)
        self.child2.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 2', 0), (u'Child 4', 1), (u'Child 5', 2), (u'Child 6', 3)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_last_child(self):
        """
        Move child2 to make it the last child of parent2
        """

        self.child2.position = -1
        self.child2.parent = Node.objects.get(pk=self.parent2.pk)
        self.child2.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2), (u'Child 2', 3)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_sibling_1(self):
        """
        Move child2 to make it the next sibling of child4
        """

        self.child2.position = 1
        self.child2.parent = Node.objects.get(pk=self.parent2.pk)
        self.child2.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 2', 1), (u'Child 5', 2), (u'Child 6', 3)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_sibling_2(self):
        """
        Move child2 to make it the next sibling of child5
        """

        self.child2.position = 2
        self.child2.parent = Node.objects.get(pk=self.parent2.pk)
        self.child2.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 2', 2), (u'Child 6', 3)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_collection_field_change_sibling_3(self):
        """
        Move child2 to make it the next sibling of child6 (last child)
        """

        self.child2.position = 3
        self.child2.parent = Node.objects.get(pk=self.parent2.pk)
        self.child2.save()

        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2), (u'Child 2', 3)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_deletion_1(self):
        """
        Delete child2
        """
        self.child2.delete()
        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 1', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_deletion_2(self):
        """
        Delete child3
        """
        self.child3.delete()
        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 2', 0), (u'Child 1', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_deletion_3(self):
        """
        Delete child1
        """
        self.child1.delete()
        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 1', 0), (u'Parent 2', 1), (u'Child 2', 0), (u'Child 3', 1), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))

    def test_deletion_4(self):
        """
        Delete parent1
        """
        self.parent1.delete()
        result = list(Node.objects.order_by('parent__position', 'position').values_list('name', 'position'))
        expected_result = [(u'Parent 2', 0), (u'Child 4', 0), (u'Child 5', 1), (u'Child 6', 2)]
        self.assertEqual(sorted(result), sorted(expected_result))


class ReorderTestCase(TestCase):
    def tearDown(self):
        Node.objects.all().delete()

    @unittest.skip("Not sure if this should fail or not. Skipping until there is time to figure it out.")
    def test_assigning_parent(self):
        a = Node.objects.create(name=u"A")
        b = Node.objects.create(name=u"B")
        c = Node.objects.create(name=u"C")
        self.assertEqual(a.position, 0)
        self.assertEqual(b.position, 1)
        self.assertEqual(c.position, 2)
        b.parent = a
        b.save()
        # A hasn't changed.
        self.assertEqual(a.position, 0)
        # B has been positioned relative to A.
        self.assertEqual(b.position, 0)
        # C has moved up to fill the gap left by B.
        self.assertEqual(c.position, 1)

    @unittest.skip("Not sure if this should fail or not. Skipping until there is time to figure it out.")
    def test_changing_parent(self):
        a = Node.objects.create(name=u"A")
        b = Node.objects.create(name=u"B")
        c = Node.objects.create(name=u"C", parent=a)
        d = Node.objects.create(name=u"D", parent=a)
        self.assertEqual(a.parent, None)
        self.assertEqual(a.position, 0)
        self.assertEqual(b.parent, None)
        self.assertEqual(b.position, 1)
        self.assertEqual(c.parent, a)
        self.assertEqual(c.position, 0)
        self.assertEqual(d.parent, a)
        self.assertEqual(d.position, 1)
        c.parent = b
        c.save()
        # A's position hasn't changed.
        self.assertEqual(a.parent, None)
        self.assertEqual(a.position, 0)
        # B's position hasn't changed.
        self.assertEqual(b.parent, None)
        self.assertEqual(b.position, 1)
        # C's relative position hasn't changed.
        self.assertEqual(c.parent, b)
        self.assertEqual(c.position, 0)
        # D has moved up to fill the gap left by C.
        self.assertEqual(d.parent, a)
        self.assertEqual(d.position, 0)


def suite():
    s = unittest.TestSuite()
    s.addTest(unittest.TestLoader().loadTestsFromTestCase(NodesTestCase))
    s.addTest(doctest.DocFileSuite(os.path.join('doctests', 'nodes.txt'), optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
    return s



from django.db.models import Manager
from django.db.models.query import QuerySet
from django.db.models.signals import post_save

from positions.fields import PositionField

import logging
logger = logging.getLogger(__name__)

class PositionQuerySet(QuerySet):
    def __init__(self, model=None, query=None, using=None, position_field_name='position', hints=None):
        super(PositionQuerySet, self).__init__(model, query, using)
        self.position_field_name = position_field_name

    def _clone(self, *args, **kwargs):
        queryset = super(PositionQuerySet, self)._clone(*args, **kwargs)
        queryset.position_field_name = self.position_field_name
        return queryset

    def reposition(self, save=True):
        position_field = self.model._meta.get_field_by_name(self.position_field_name)[0]
        post_save.disconnect(position_field.update_on_save, sender=self.model)
        position = 0
        for obj in self.iterator():
            setattr(obj, self.position_field_name, position)
            if save:
                obj.save()
            position += 1
        post_save.connect(position_field.update_on_save, sender=self.model)
        return self


class PositionManager(Manager):
    def __init__(self, position_field_name='position'):
        super(PositionManager, self).__init__()
        self.position_field_name = position_field_name

    def get_queryset(self):
        return PositionQuerySet(self.model, position_field_name=self.position_field_name)

    def get_query_set(self):
        return self.get_queryset(self.model, position_field_name=self.position_field_name)

    def reposition(self):
        return self.get_queryset().reposition()



from unittest import skipIf
from models import MigrationTest
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
import django

class MigrationTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    @skipIf(django.VERSION < (1,7), 'Skipping migration test because Django < 1.7')
    def test_migration(self):
        # The data migration should have inserted the following record. This test just verifies that the data is there.
        # Ideally this test would run the migrations but setting up a data migration is faster for now.
        test = MigrationTest.objects.create(name="Some Person", age=37, favorite_color='Blue')
        result = list(MigrationTest.objects.order_by('position').values_list('name', 'age', 'favorite_color', 'position'))
        expected_result = [(u'Test Name', 99, u'Red', -1), (u'Some Person', 37, u'Blue', 0)]
        self.assertEqual(result, expected_result)


import doctest
import unittest

from django.contrib.contenttypes.models import ContentType

from positions.examples.lists.models import List
from positions.examples.generic.models import GenericThing

from django.test import TestCase

class GenericTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        GenericThing.objects.all().delete()
        ContentType.objects.all().delete()
        List.objects.all().delete()

    # @unittest.skip("Some reason. If you are reading this in a test run someone did not fill this in.")
    def test_doctests_standin(self):
        # This code just contains the old doctests for this module. They should be most likely split out into their own
        # tests at some point.
        self.l = List.objects.create(name='To Do')
        self.ct = ContentType.objects.get_for_model(self.l)
        self.t1 = GenericThing.objects.create(name="First Generic Thing", object_id=self.l.pk, content_type=self.ct)

        self.t2 = GenericThing.objects.create(name="Second Generic Thing", object_id=self.l.pk, content_type=self.ct)
        self.assertEquals(self.t1.position, 0)
        self.assertEquals(self.t2.position, 1)
        self.t1.position = 1
        self.t1.save()

        self.assertEquals(self.t1.position, 1)
        self.t2 = GenericThing.objects.get(pk=2)
        self.assertEquals(self.t2.position, 0)
        self.t1.delete()

        actual_order = list(GenericThing.objects.filter(object_id=self.l.pk, content_type=self.ct).values_list('name', 'position').order_by('position'))
        expected_order = [(u'Second Generic Thing', 0)]
        self.assertEqual(actual_order, expected_order)
        self.t3 = GenericThing.objects.create(object_id=self.l.pk, content_type=self.ct, name='Mr. None')
        self.t3.save()
        self.assertEquals(self.t3.position, 1)
        self.t4 = GenericThing.objects.create(object_id=self.l.pk, content_type=self.ct, name='Mrs. None')
        self.assertEquals(self.t4.position, 2)
        self.t4.position = -2
        self.t4.save()
        self.assertEquals(self.t4.position, 1)
        actual_order = list(GenericThing.objects.order_by('position').values_list('name', flat=True))
        expected_order = [u'Second Generic Thing', u'Mrs. None', u'Mr. None']
        self.assertEqual(actual_order, expected_order)



from django.test import TestCase
import doctest
import unittest

from django.db import models

from positions.examples.school.models import SubUnit, Lesson, Exercise


class SchoolsTestCase(TestCase):
    def setUp(self):
        self.american_revolution = SubUnit.objects.create(name="American Revolution")
        self.no_taxation = Lesson.objects.create(sub_unit=self.american_revolution, title="No Taxation without Representation", text="...")
        self.assertEqual(self.no_taxation.position, 0)

        self.research_paper = Exercise.objects.create(sub_unit=self.american_revolution, title="Paper", description="Two pages, double spaced")
        self.assertEqual(self.research_paper.position, 1)

        self.tea_party = Lesson.objects.create(sub_unit=self.american_revolution, title="Boston Tea Party", text="...")
        self.assertEqual(self.tea_party.position, 2)

        self.quiz = Exercise.objects.create(sub_unit=self.american_revolution, title="Pop Quiz", description="...")
        self.assertEqual(self.quiz.position, 3)

    def tearDown(self):
        SubUnit.objects.all().delete()

    @unittest.skip("This should not fail! Skipping during test development.")
    def test_explicit_position(self):
        # create a task with an explicit position
        self.intro_lesson = Lesson.objects.create(sub_unit=self.american_revolution, title="The Intro", text="...", position=0)
        actual_order = list(self.american_revolution.task_set.values_list('title', 'position'))
        expected_order = [
            (u'The Intro', 0),
            (u'No Taxation without Representation', 1),
            (u'Paper', 2),
            (u'Boston Tea Party', 3),
            (u'Pop Quiz', 4)
        ]
        self.assertEqual(actual_order, expected_order)


from .settings import *

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'django_positions',
        'USER': 'django_positions',
        'PASSWORD': 'django_positions',
    }
}

LOGGING['handlers']['debug_log_file']['formatter'] = 'simple'



from django.db import models
from positions.fields import PositionField

class Node(models.Model):
    parent = models.ForeignKey('self', related_name='children', blank=True, null=True)
    name = models.CharField(max_length=50)
    position = PositionField(collection='parent')

    def __unicode__(self):
       return self.name
